package org.testinfinispan.util;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;

import org.infinispan.manager.CacheContainer;
import org.infinispan.manager.EmbeddedCacheManager;
import org.jboss.util.naming.NonSerializableFactory;
import org.jnp.server.Main;
import org.jnp.server.SingletonNamingServer;

public class TestJndi {
    private static Main namingMain;
    private static SingletonNamingServer namingServer;
    private static Properties props;
    /*public void startJndiServer() throws Exception {
        // Create an in-memory jndi
        namingServer = new SingletonNamingServer();
        namingMain = new Main();
        namingMain.setInstallGlobalService(true);
        namingMain.setPort(-1);
        namingMain.start();
        props = new Properties();
        props.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
        props.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
     }*/
    
    static {
        // Create an in-memory jndi
        try {
			namingServer = new SingletonNamingServer();
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        namingMain = new Main();
        namingMain.setInstallGlobalService(true);
        namingMain.setPort(-1);
        try {
			namingMain.start();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        props = new Properties();
        props.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
        props.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
     }
    
    public static void bindCacheManagerToJndi(EmbeddedCacheManager cacheManager) throws Exception {
        Context ctx = new InitialContext(props);
        String jndiName = "java:CacheManager/entity";
        bind(jndiName, cacheManager, CacheContainer.class, ctx);
        ctx.lookup(jndiName);
        /*try {
           Context ctx2 = new InitialContext(props);
           try {
              EmbeddedCacheManager cacheManager2 = (EmbeddedCacheManager) ctx2.lookup(jndiName);
              assert cacheManager.getStatus() == cacheManager2.getStatus();
           } finally {
              ctx2.close();
           }
        } finally {
           /*unbind(jndiName, ctx);
           ctx.close();
        }*/
     }
    private static void bind(String jndiName, Object who, Class<?> classType, Context ctx) throws Exception {
        // Ah ! This service isn't serializable, so we use a helper class
        NonSerializableFactory.bind(jndiName, who);
        Name n = ctx.getNameParser("").parse(jndiName);
        while (n.size() > 1) {
           String ctxName = n.get(0);
           try {
              ctx = (Context) ctx.lookup(ctxName);
           } catch (NameNotFoundException e) {
              //log.debug("creating Subcontext " + ctxName);
              ctx = ctx.createSubcontext(ctxName);
           }
           n = n.getSuffix(1);
        }

        // The helper class NonSerializableFactory uses address type nns, we go on to
        // use the helper class to bind the service object in JNDI
        StringRefAddr addr = new StringRefAddr("nns", jndiName);
        Reference ref = new Reference(classType.getName(), addr, NonSerializableFactory.class.getName(), null);
        ctx.rebind(n.get(0), ref);
     }

    private static void unbind(String jndiName, Context ctx) throws Exception {
        NonSerializableFactory.unbind(jndiName);
        ctx.unbind(jndiName);
     }
}
